from openpyxl.writer.excel import ExcelWriter
from openpyxl.xml.constants import SHEET_MAIN_NS, ARC_SHARED_STRINGS
from openpyxl.xml.functions import Element, xmlfile, SubElement
from openpyxl.cell.text import Text
from io import BytesIO

PRESERVE_SPACE = '{%s}space' % "http://www.w3.org/XML/1998/namespace"


def write_string_table(string_table):
    """Write the string table xml."""
    out = BytesIO()

    with xmlfile(out) as xf:
        with xf.element("sst", xmlns=SHEET_MAIN_NS, uniqueCount="%d" % len(string_table)):

            for key in string_table:
                if isinstance(key, Text) and key.formatted:
                    xf.write(key.to_tree(tagname='si'))
                    continue
                el = Element('si')
                text = SubElement(el, 't')
                text.text = key
                if key.strip() != key:
                    text.set(PRESERVE_SPACE, 'preserve')
                xf.write(el)

    return out.getvalue()


class SynoExcelWriter(ExcelWriter):

    def write_data(self):
        ExcelWriter.write_data(self)
        self._archive.writestr(ARC_SHARED_STRINGS,
                               write_string_table(self.workbook.shared_strings))
